/*
Copyright (C) 2013 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
 */

package com.clustercontrol.cloud.ui.views.actions;


import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;

import com.clustercontrol.cloud.Activator;
import com.clustercontrol.cloud.commons.util.ControlUtil;
import com.clustercontrol.cloud.commons.util.MessageManager;
import com.clustercontrol.cloud.presenter.CreateCloudUserRequest;
import com.clustercontrol.cloud.presenter.IAccountResource;
import com.clustercontrol.cloud.presenter.IAccountResourceManager;
import com.clustercontrol.cloud.presenter.ICloudUserManager;
import com.clustercontrol.cloud.ui.dialogs.CreateCloudUserDialog;
import com.clustercontrol.util.Messages;


public class CreateCloudUserAction extends AbstractViewAction {
	public static final String ID = "com.clustercontrol.cloud.ui.views.actions.CreateCloudUserAction";

	private MessageManager messages = MessageManager.getInstance("messages");
	
	private String msgCreateCloudUserConfirm = messages.getString("message.confirm_some_action_to_something", new Object[]{"word.cloud_user", "word.create"});
	private String msgCreateCloudUserResult = messages.getString("message.result_some_action_to_something", new Object[]{"word.cloud_user", "word.create"});

	@Override
	protected void invoke(IAction action) throws InvocationException {
		
		CreateCloudUserDialog dialog = new CreateCloudUserDialog(this.viewPart.getSite().getShell());

		if (dialog.open() == Window.OK) {
			if (MessageDialog.openConfirm(
				null,
				Messages.getString("confirmed"),
				msgCreateCloudUserConfirm)) {
				
				CreateCloudUserDialog.DialogOutput output = dialog.getOutput();
				
				CreateCloudUserRequest request = new CreateCloudUserRequest(); 
				ControlUtil.importObject(request, output);
				
				IAccountResourceManager accountResourceManager = Activator.getDefault().getCloudResourceManager().getAccountResourceManager();
				IAccountResource accountResource = accountResourceManager.getAccountResource(output.getAccountResourceId());
				ICloudUserManager manager = accountResource.getCloudUserManager();
				
				manager.createCloudUser(request);
				
				// 成功報告ダイアログを生成
				MessageDialog.openInformation(
					null,
					Messages.getString("successful"),
					msgCreateCloudUserResult);
			}
			else {
				return;
			}
		}
	}
}